"use strict";
/*!
 * planisphere
 *
 * A straightforward sitemap generator written in TypeScript.
 *
 * Copyright (c) 2020-present, cheap glitch
 *
 * Permission  to use,  copy, modify,  and/or distribute  this software  for any
 * purpose  with or  without  fee is  hereby granted,  provided  that the  above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS  SOFTWARE INCLUDING ALL IMPLIED  WARRANTIES OF MERCHANTABILITY
 * AND FITNESS. IN NO EVENT SHALL THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT, OR CONSEQUENTIAL  DAMAGES OR ANY DAMAGES  WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER  TORTIOUS ACTION,  ARISING OUT  OF  OR IN  CONNECTION WITH  THE USE  OR
 * PERFORMANCE OF THIS SOFTWARE.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSitemaps = exports.generateSitemapsIndex = exports.writeSitemaps = void 0;
const path_1 = require("path");
const promises_1 = require("fs/promises");
const XML_DOCTYPE = '<?xml version="1.0" encoding="UTF-8"?>';
const MAX_NB_URLS_IN_SITEMAP = 50000;
async function writeSitemaps(destination, urls, options = {}) {
    const sitemaps = generateSitemaps(urls, options);
    if (sitemaps.length === 0) {
        return;
    }
    const sitemapFilenames = (sitemaps.length === 1) ? ['sitemap.xml'] : sitemaps.map((_, index) => `sitemap-part-${(index + 1).toString().padStart(2, '0')}.xml`);
    await Promise.all(sitemaps.map((sitemap, index) => promises_1.writeFile(path_1.resolve(destination, sitemapFilenames[index]), sitemap)));
    const sitemapIndex = generateSitemapsIndex(sitemapFilenames, options);
    if (sitemapIndex !== undefined) {
        await promises_1.writeFile(path_1.resolve(destination, 'sitemap-index.xml'), sitemapIndex);
    }
}
exports.writeSitemaps = writeSitemaps;
function generateSitemapsIndex(sitemapFilenames, options = {}, lastmod = new Date()) {
    var _a;
    if (sitemapFilenames.length <= 1) {
        return undefined;
    }
    const baseUrl = options.baseUrl ? options.baseUrl.replace(/\/+$/, '') : '';
    const pretty = (_a = options.pretty) !== null && _a !== void 0 ? _a : false;
    const NL = pretty ? '\n' : '';
    const TAB = pretty ? '\t' : '';
    const sitemaps = sitemapFilenames.map(filename => {
        return TAB + xmlTag('sitemap', NL +
            TAB + TAB + xmlTag('loc', [baseUrl, filename].filter(Boolean).join('/')) + NL +
            TAB + TAB + xmlTag('lastmod', formatUrlLastmod(lastmod)) + NL + TAB);
    });
    return XML_DOCTYPE + NL + '<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' + NL + sitemaps.join(NL) + NL + '</sitemapindex>';
}
exports.generateSitemapsIndex = generateSitemapsIndex;
function generateSitemaps(urls, options = {}) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    const baseUrl = options.baseUrl ? options.baseUrl.replace(/\/+$/, '') : '';
    const trailingSlash = (_a = options.trailingSlash) !== null && _a !== void 0 ? _a : undefined;
    const pretty = (_b = options.pretty) !== null && _b !== void 0 ? _b : false;
    const NL = pretty ? '\n' : '';
    const TAB = pretty ? '\t' : '';
    const sitemaps = [];
    for (let offset = 0; offset < Math.ceil(urls.length / MAX_NB_URLS_IN_SITEMAP); offset++) {
        const sitemapUrls = [];
        for (let i = offset * MAX_NB_URLS_IN_SITEMAP; i < (offset + 1) * MAX_NB_URLS_IN_SITEMAP && i < urls.length; i++) {
            const url = (typeof urls[i] === 'string' ? { loc: urls[i] } : urls[i]);
            let loc = [baseUrl, url.loc.replace(/^\//, '')].filter(Boolean).join('/');
            if (trailingSlash === true && !loc.endsWith('/')) {
                loc += '/';
            }
            else if (trailingSlash === false && loc.endsWith('/')) {
                loc = loc.slice(0, -1);
            }
            const lastmod = ('lastmod' in url) ? url.lastmod : ((_d = (_c = options.defaults) === null || _c === void 0 ? void 0 : _c.lastmod) !== null && _d !== void 0 ? _d : undefined);
            const changefreq = ('changefreq' in url) ? url.changefreq : ((_f = (_e = options.defaults) === null || _e === void 0 ? void 0 : _e.changefreq) !== null && _f !== void 0 ? _f : undefined);
            const priority = ('priority' in url) ? url.priority : ((_h = (_g = options.defaults) === null || _g === void 0 ? void 0 : _g.priority) !== null && _h !== void 0 ? _h : undefined);
            sitemapUrls.push(TAB + xmlTag('url', NL +
                TAB + TAB + xmlTag('loc', formatUrlLoc(loc)) + NL +
                (lastmod !== undefined ? TAB + TAB + xmlTag('lastmod', formatUrlLastmod(lastmod)) + NL : '') +
                (changefreq !== undefined ? TAB + TAB + xmlTag('changefreq', changefreq) + NL : '') +
                (priority !== undefined ? TAB + TAB + xmlTag('priority', formatUrPriority(priority)) + NL : '') + TAB));
        }
        sitemaps.push(XML_DOCTYPE + NL +
            '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' + NL +
            sitemapUrls.join(NL) + NL +
            '</urlset>');
    }
    return sitemaps;
}
exports.generateSitemaps = generateSitemaps;
function formatUrlLoc(loc) {
    const replacements = {
        '&': 'amp',
        "'": 'apos',
        '"': 'quot',
        '<': 'lt',
        '>': 'gt',
    };
    return encodeURI(loc).replace(/["&'<>]/g, character => '&' + replacements[character] + ';');
}
function formatUrlLastmod(lastmod) {
    return ((lastmod instanceof Date) ? lastmod : new Date(lastmod)).toISOString();
}
function formatUrPriority(priority) {
    if (priority === 0) {
        return '0.0';
    }
    if (priority === 1) {
        return '1.0';
    }
    return priority.toString();
}
function xmlTag(tag, contents) {
    return `<${tag}>${contents}</${tag}>`;
}
